﻿using FLIPilotTakeAPic.ViewModel;
using System;
using System.Windows;
using System.Windows.Interop;
using System.Diagnostics;
using System.Windows.Forms;

namespace FLIPilotTakeAPic
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        //**************************************************************
        #region Private Constants

        private const int CONTROL_WIDTH_STANDARD = 370;
        private const int CONTROL_WIDTH_SEPARATED = 700;
        private const double WINDOW_PANEL_WIDTH_LEFT = 0.33;
        private const double WINDOW_PANEL_WIDTH_RIGHT = 0.66;
        private const double WINDOW_PANEL_ROW_HEIGHT = 0.5;

        #endregion Private Constants

        //**************************************************************
        #region Private Variables

        private MainWindowViewModel mwvm;

        #endregion Private Variables

        //**************************************************************
        #region Constructor and Initialization

        public MainWindow()
        {
            try
            {
                string s = Process.GetCurrentProcess().ProcessName;
                if (Process.GetProcessesByName(Process.GetCurrentProcess().ProcessName).Length > 1)
                {
                    System.Windows.MessageBox.Show("Application FLIPilot already running. Only one instance of this application is allowed");
                    System.Windows.Application.Current.Shutdown();
                }
                else
                {
                    InitializeComponent();

                    mwvm = new MainWindowViewModel();

                    this.DataContext = new
                    {
                        MWVM = mwvm,
                        This = this
                    };
                }
            }
            catch (Exception ex)
            {
                System.Windows.MessageBox.Show("Error encountered starting application:\n\n" + ex.Message + "\n\n" + ex.InnerException.Message + "\n\n" + ex.StackTrace);
            }
        }

        #endregion Constructor and Initialization

        //**************************************************************
        #region Observable Properties

        #endregion Observable Properties

        //**************************************************************
        #region Private Methods

        #endregion Private Methods

        //**************************************************************
        #region Click Functions

        private void Browse_Click(object sender, RoutedEventArgs e)
        {
            using (FolderBrowserDialog dlg = new FolderBrowserDialog())
            {
                dlg.Description = "Target Raw File Folder";
                dlg.ShowNewFolderButton = true;
                dlg.SelectedPath = mwvm.RawFilePath;
                DialogResult result = dlg.ShowDialog();
                if (result == System.Windows.Forms.DialogResult.OK)
                {
                    mwvm.RawFilePath = dlg.SelectedPath;
                }
            }
            txtRawFilePath.Focus();
            txtRawFilePath.CaretIndex = txtRawFilePath.Text.Length;
        }

        /// <summary>
        /// Starts the selected Grab operation using the Image Coordinates
        /// </summary>
        public void Grab_Click(object sender, RoutedEventArgs e)
        {
            mwvm.GrabClick();
        }

        #endregion Click Functions

        //**************************************************************
        #region Window Functions

        #endregion Window Functions

        //**************************************************************
        #region WinProc Functions

        /// <summary>
        /// AddHook Handle WndProc messages in WPF
        /// This cannot be done in a Window's constructor as a handle window handle won't at that point, so there won't be a HwndSource.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnSourceInitialized(EventArgs e)
        {
            base.OnSourceInitialized(e);

            var hwndSource = PresentationSource.FromVisual(this) as HwndSource;
            if (hwndSource != null)
            {
                hwndSource.AddHook(WndProc);
                staticThis = this;
            }
        }

        private static MainWindow staticThis;

        const int WM_DESTROY = 0x02;
        const int WM_DEVICECHANGE = 0x219;

        enum DBT
        {
            DEVICEARRIVAL = 0x8000,
        }

        /// <summary>
        /// WndProc matches the HwndSourceHook delegate signature so it can be passed to AddHook() as a callback. 
        /// This is the same as overriding a Windows.Form's WncProc method.
        /// </summary>
        /// <param name="hwnd">The window handle</param>
        /// <param name="msg">The message ID</param>
        /// <param name="wParam">The message's wParam value, historically used in the win32 api for handles and integers</param>
        /// <param name="lParam">The message's lParam value, historically used in the win32 api to pass pointers</param>
        /// <param name="handled">A value that indicates whether the message was handled</param>
        /// <returns></returns>
        private IntPtr WndProc(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
        {
            if (msg == WM_DESTROY)
            {
                var hwndSource = PresentationSource.FromVisual(staticThis) as HwndSource;
                if (hwndSource != null)
                {
                    hwndSource.RemoveHook(WndProc);
                }
            }

            return IntPtr.Zero;
        }

        #endregion WinProc Functions

    }
}
