﻿using FliUtils.Mvvm;
using FliProCameraLib;
using System;
using System.Collections.Generic;
using System.Drawing;
//using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using static FliProCameraLib.FliPro;

namespace FLIPilotSimpleViewModel
{
    public class GrabPanelViewModel : ObservableProperties
    {

        //**************************************************************
        #region Constructor and Initialization

        public GrabPanelViewModel()
        {
            fliProCameraManager = FliProCameraManager.Instance;
        }

        #endregion Constructor and Initialization

        //**************************************************************
        #region Constants

        public const string GRAB_BUTTON_TITLE_STOP_GRAB = "Stop Grab";
        public const string GRAB_BUTTON_TITLE_GRAB_START = "Grab ";
        public const string GRAB_BUTTON_TITLE_GRAB_END = " Frame";

        public const int CONTINUOUS_PROGRESS_BAR_COUNT = 100;

        #endregion Constants

        //**************************************************************
        #region Private Variable

        private FliProCameraManager fliProCameraManager;
        static AutoResetEvent autoEvent = new AutoResetEvent(false);
        private bool WaitingForGrabToComplete = false;
        // Attach a handler to the frame queue
        private ControlPanelViewModel cpvm;

        #endregion Private Variable

        //**************************************************************
        #region Observable Properties

        public bool GrabButtonIsEnabled
        {
            get { return _GrabButtonIsEnabled; }
            set { SetValue<bool>(ref _GrabButtonIsEnabled, value); }
        }
        protected bool _GrabButtonIsEnabled = true;

        public string GrabButtonTitle
        {
            get { return _GrabButtonTitle; }
            set { SetValue<string>(ref _GrabButtonTitle, value); }
        }
        protected string _GrabButtonTitle = GRAB_BUTTON_TITLE_GRAB_START + GRAB_BUTTON_TITLE_GRAB_END;

        public int RestartCounter
        {
            get { return _RestartCounter; }
            set { SetValue<int>(ref _RestartCounter, value); }
        }
        protected int _RestartCounter = 0;

        #endregion Observable Properties

        //**************************************************************
        #region Private Functions

        private void SetButtonStates()
        {
            if (GrabButtonTitle.Equals(GRAB_BUTTON_TITLE_STOP_GRAB))
            {
                fliProCameraManager.DisplayCamera.SetEnabled(false);
            }
            else
            {
                fliProCameraManager.DisplayCamera.SetEnabled(true);
            }
            OnPropertyChanged("");
        }

        /// <summary>
        /// This function is used to trigger a new StartGrab as soon as the previous Grab has completed.
        /// </summary>
        private void StartGrabThread()
        {
            while (fliProCameraManager.ActiveCamera.IsCapturing)
            {
                autoEvent.WaitOne(10);
            }

            if (!fliProCameraManager.ActiveCamera.IsCapturing)
            {
                StartGrab(false, 0);
            }
        }

        #endregion Private Functions

        //**************************************************************
        #region Public Functions

        public string GetGrabTitle()
        {
            return GRAB_BUTTON_TITLE_GRAB_START +
                   fliProCameraManager.ActiveCamera.FliFrameTypeList.Where(x => x.Value == fliProCameraManager.ActiveCamera.FrameType).First().Name +
                   GRAB_BUTTON_TITLE_GRAB_END + (((fliProCameraManager.ActiveCamera.FrameCountType == FliFrameCount.MULTIPLE) ||
                   fliProCameraManager.ActiveCamera.FrameCountType == FliFrameCount.CONTINUOUS) ? "s" : String.Empty);
        }

        public void GrabButtonsEnabled(bool enabled)
        {
            GrabButtonIsEnabled = enabled;
            fliProCameraManager.ActiveCamera.SetEnabled(enabled);
        }

        public void GrabClick()
        {
            if (GrabButtonTitle.Equals(GrabPanelViewModel.GRAB_BUTTON_TITLE_STOP_GRAB))
            {
                GrabButtonsEnabled(false);
                OnPropertyChanged("GrabButtonIsEnabled");
                fliProCameraManager.ActiveCamera.CancelExposure();
            }
            else if (fliProCameraManager.ActiveCamera.IsConnected)
            {
                fliProCameraManager.ActiveCamera.FastThroughput = false;
                fliProCameraManager.ActiveCamera.ProcessedFrameThroughputCount = 0;

                fliProCameraManager.PublishStatus(GetGrabTitle() + " ... ", false);
                fliProCameraManager.FramesCaptured = 0;
                fliProCameraManager.ActiveCamera.IsAborted = false;

                StartGrab(true, fliProCameraManager.ActiveCamera.FrameCount);
            }
            else
            {
                MessageBox.Show("Please select a connected camera");
                fliProCameraManager.ActiveCamera.IsAborted = true;
                fliProCameraManager.OnActiveJobCompleteAction(new FliProCameraManager.ActiveJobCompleteEventArgs(!fliProCameraManager.ActiveCamera.IsAborted, null));
            }
        }

        public void SetDataContext(ControlPanelViewModel vm)
        {
            cpvm = vm;
        }

        public void StartGrab(bool firstCall, int frameCount)
        {
            try
            {
                if (fliProCameraManager.ActiveIndex < 0)
                {
                    MessageBox.Show("There are no Cameras connected.");
                }
                else
                {
                    fliProCameraManager.SetActiveCameraToForefront();
                    if (!fliProCameraManager.ActiveCamera.IsConnected)
                    {
                        MessageBox.Show("Camera " + fliProCameraManager.ActiveCamera.SerialNumber +
                            " is no longer connected.\nSelect a Camera from the drop list that is connected.");
                    }
                    else
                    {
                        //
                        // Call Simple Grab
                        StartGrabSimple();
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error in StartGrab:\n" + ex.Message + "\n\n" + ex.StackTrace, "Warning", MessageBoxButton.OK,
                    MessageBoxImage.Warning, MessageBoxResult.OK, MessageBoxOptions.DefaultDesktopOnly);
            }
        }

        public void StartGrabSimple()
        {
            // Setup the tool stuff
            fliProCameraManager.SetActiveCameraToForefront();
            GrabButtonTitle = GRAB_BUTTON_TITLE_STOP_GRAB;
            SetButtonStates();

            // This function gets the image from the camera directly
            FliFrame frame = fliProCameraManager.ActiveCamera.GetCompletedFrame();

            if ((frame != null) && frame.IsValid)
            {
                // Only write valid frames here. Invalid frames are written in ParseRawImageData
                fliProCameraManager.ActiveCamera.WriteFileBasic(frame, fliProCameraManager.RawFilePath);
            }

            // Clean up the tool stuff
            GrabButtonsEnabled(true);
            GrabButtonTitle = GetGrabTitle();
            fliProCameraManager.PublishStatus((frame.IsValid == false ? "Failed" : "Done"), true);
            SetButtonStates();
            fliProCameraManager.StatusProgressFinished();
            fliProCameraManager.OnActiveJobCompleteAction(new FliProCameraManager.ActiveJobCompleteEventArgs(frame.IsValid != false, frame));
        }

        #endregion Public Functions

    }
}
