﻿using FliUtils.Jobs;
using FliUtils.Mvvm;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;

namespace FliProCameraLib
{
    //-=-= FIXME: This needs to be changed to use ConsumerJob
    public class FliFrameQueue
    {
        //**************************************************************
        #region Constructor
        private FliFrameQueue()
        {
            // This was for Debug
            //FrameQueueJob = new FrameQueueClass(NotifyFrameEnqueued);

            FrameQueueJob = new ConsumerJob<FliFrame>(NotifyFrameEnqueued);
            FrameQueueJob.Start();
            //FrameQueueProcessingJob = new ConsumerJob<FliFrame>(NotifyProcessingFrameEnqueued);
            //FrameQueueProcessingJob.Start();
        }

        #endregion Constructor

        //**************************************************************
        #region Public Types

        public class FrameQueueEventArgs : EventArgs
        {
            public FrameQueueEventArgs() { }

            public FrameQueueEventArgs(FliFrame frame, bool processed)
            {
                Frame = frame;
                Processed = processed;
            }
            public FliFrame Frame { get; set; }
            public bool Processed { get; set; }
        }

        #endregion Public Types

        //**************************************************************
        #region Private Classes

        // This was for Debug
        //private class FrameQueueClass
        //{
        //    public FrameQueueClass(NotifyFrameFunction consumeOne)
        //    {
        //        ConsumeOne = consumeOne;
        //        _thread = new Thread(new ThreadStart(ConsumerThread));
        //        _thread.Priority = ThreadPriority.Highest;
        //        _thread.Start();
        //    }

        //    static AutoResetEvent autoEvent = new AutoResetEvent(false);
        //    private List<FliFrame> FrameList = new List<FliFrame>();
        //    private NotifyFrameFunction ConsumeOne;
        //    private Thread _thread;

        //    // Debug code
        //    private DateTime autoEventSetTime;
        //    private FliProCameraManager fliProCameraManager;
        //    private void PostStatus(string CameraID, string text, TimeSpan timeInQueue)
        //    {
        //        if (fliProCameraManager == null)
        //        {
        //            fliProCameraManager = FliProCameraManager.Instance;
        //        }
        //        FliProCamera camera = fliProCameraManager.GetFliProCamera(CameraID);
        //        camera.ExposureStatusText = text + timeInQueue.Seconds.ToString("00") + "." + timeInQueue.Milliseconds.ToString("000");
        //        camera.LogEvent(text + timeInQueue.Seconds.ToString("00") + "." + timeInQueue.Milliseconds.ToString("000"));
        //    }


        //    public int QueueCount { get { return FrameList.Count; } }

        //    public delegate void NotifyFrameFunction(FliFrame frame);
        //    public delegate void PostStatusFunction(string CameraID, string text, TimeSpan timeInQueue);

        //    public void Enqueue(FliFrame frame)
        //    {
        //        FrameList.Add(frame);
        //        autoEvent.Set();
        //        autoEventSetTime = DateTime.Now;
        //    }

        //    private void ConsumerThread()
        //    {
        //        FliFrame frame = null;

        //        // Loop forever
        //        while (true)
        //        {
        //            try
        //            {
        //                while (FrameList.Count == 0)
        //                {
        //                    Thread.Sleep(10);
        //                }

        //                // Debug Code
        //                DateTime dequeueTime = DateTime.Now;

        //                if (FrameList.Count > 0)
        //                {
        //                    frame = FrameList[FrameList.Count - 1];

        //                    // Debug Code
        //                    PostStatus(frame.CameraID, "Time for autoEvent Notify: ", dequeueTime - autoEventSetTime);

        //                    FrameList.RemoveAt(FrameList.Count - 1);
        //                }

        //                // Debug Code
        //                PostStatus(frame.CameraID, "Time for removal from list: ", DateTime.Now - dequeueTime);

        //                ConsumeOne(frame);
        //            }
        //            catch (Exception ex)
        //            {
        //                string str = ex.Message;
        //            }
        //        }
        //    }

        //}

        #endregion Private Classes

        //**************************************************************
        #region Private Properties

        private static volatile FliFrameQueue _Instance;
        private static object _SyncRoot = new Object();
        protected ConsumerJob<FliFrame> FrameQueueJob { get; set; }
        protected ConsumerJob<FliFrame> FrameQueueProcessingJob { get; set; }

        // This was for Debug
        //private  FrameQueueClass FrameQueueJob { get; set; }

        private FliProCameraManager fliProCameraManager;

        #endregion Private Properties

        //**************************************************************
        #region Private Methods

        private void NotifyFrameEnqueued(FliFrame frame)
        {
            bool frameProcessed = true;
            FliProCamera camera = frame.Camera;
            frame.RemoveTime = DateTime.Now;
            frame.TimeInQueue = frame.RemoveTime - frame.InsertTime;
            if (fliProCameraManager == null)
            {
                fliProCameraManager = FliProCameraManager.Instance;
            }
            if (camera == null)
            {
                camera = fliProCameraManager.GetFliProCamera(frame.CameraID);
            }

            try
            {
                // NOTE: This code was for an idea to queue multiple jobs and return quickly
                //if (camera.FastThroughput && frame.IsValid)
                //{
                //    camera.LogEvent("NotifyFrameEnqueued: Enqueuing frame for processing");

                //    // Put this frame in the Processing
                //    FrameQueueProcessingJob.Enqueue(frame);
                //    // Start the next frame
                //    OnFrameQueueComplete(new FrameQueueEventArgs(frame));
                //    frameProcessed = false;
                //}
                //else
                //{

                // If the frame is not valid we want to process it now so we can restart MULTIPLE and CONTINUOUS FrameTypes
                camera.LogEvent("NotifyFrameEnqueued: calling frame.ParseRawImageData");
                frameProcessed = frame.ParseRawImageData(camera, FrameQueueJob.QueueCount);

                //}
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error in NotifyFrameEnqueued:" + ex.Message, "Warning", MessageBoxButton.OK,
                    MessageBoxImage.Warning, MessageBoxResult.OK, MessageBoxOptions.DefaultDesktopOnly);
            }

            // Need to call this after every frame
            camera.LogEvent("NotifyFrameEnqueued: calling OnFrameQueueComplete");
            OnFrameQueueComplete(new FrameQueueEventArgs(frame, frameProcessed));

            // Need these here so that the buttons on the screen are reset even if an error is raised
            if (frameProcessed)
            {
                bool needToProcess = (((camera.IsSaveExposureToBINFileChecked || camera.IsSaveExposureToFITFileChecked ||
                    camera.IsSaveExposureToJPEGFileChecked || camera.IsSaveExposureToPNGFileChecked ||
                    camera.IsSaveExposureToRAWFileChecked || camera.IsSaveExposureToRCDFileChecked ||
                    camera.IsSaveExposureToTIFFFileChecked) &&
                    (camera.UseHighGainImageChecked ||camera.UseHighGainImageChecked || camera.UseMergedImageChecked)) || camera.IsSaveMetaDataToFileChecked ||
                    fliProCameraManager.TestRoutineActive);
                camera.AddFrame(frame, (needToProcess ? FliProCamera.PurgeActionEnum.WaitPurge : FliProCamera.PurgeActionEnum.ForcePurge));
                camera.CurrentFrame = frame;
                camera.CurrentFrameIndex = 1;
                OnFrameQueueAction(new FrameQueueEventArgs(frame, frameProcessed));
            }

            camera.LogEvent("NotifyFrameEnqueued complete");
        }

        private void NotifyProcessingFrameEnqueued(FliFrame frame)
        {
            // For debug
            FliProCamera camera;
            if (fliProCameraManager == null)
            {
                fliProCameraManager = FliProCameraManager.Instance;
            }
            camera = fliProCameraManager.GetFliProCamera(frame.CameraID);
            camera.LogEvent("NotifyProcessingFrameEnqueued called with new Frame. QueueCount = " + FrameQueueJob.QueueCount.ToString());

            bool processed = false;

            try
            {
                processed = frame.ParseRawImageData(camera, FrameQueueJob.QueueCount);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error in NotifyProcessingFrameEnqueued:" + ex.Message, "Warning", MessageBoxButton.OK,
                    MessageBoxImage.Warning, MessageBoxResult.OK, MessageBoxOptions.DefaultDesktopOnly);
                processed = true;
            }


            if (processed)
            {
                // Need this here so that the buttons on the screen are reset even if an error is raised
                OnFrameQueueAction(new FrameQueueEventArgs(frame, processed));
            }

            // For debug
            camera.LogEvent("NotifyProcessingFrameEnqueued complete");
        }

        private void PostStatus(string CameraID, string text, TimeSpan timeInQueue)
        {
            if (fliProCameraManager == null)
            {
                fliProCameraManager = FliProCameraManager.Instance;
            }
            FliProCamera camera = fliProCameraManager.GetFliProCamera(CameraID);
            //camera.ExposureStatusText = text + timeInQueue.Seconds.ToString("00") + "." + timeInQueue.Milliseconds.ToString("000");
            camera.LogEvent(text + timeInQueue.Seconds.ToString("00") + "." + timeInQueue.Milliseconds.ToString("000"));
        }
        #endregion Private Methods

        //**************************************************************
        #region Public Properties

        //-------------------------------------------
        /// <summary>
        /// Returns a reference to the single instance of this class.
        /// </summary>
        public static FliFrameQueue Instance
        {
            get
            {
                if (_Instance == null)
                {
                    lock (_SyncRoot)
                    {
                        if (_Instance == null)
                        {
                            _Instance = new FliFrameQueue();
                        }
                    }
                }

                return _Instance;
            }
        }
        
        public int Count { get { return FrameQueueJob.QueueCount; } }

        #endregion Public Properties

        //**************************************************************
        #region Public Methods

        public void Close()
        {
            FrameQueueJob.Stop();
        }

        public void InsertFrame(FliFrame frame)
        {
            // For debugging
            //if (fliProCameraManager == null)
            //{
            //    fliProCameraManager = FliProCameraManager.Instance;
            //}
            //FliProCamera camera = fliProCameraManager.GetFliProCamera(frame.CameraID);
            //camera.LogEvent("InsertFrame called");

            frame.InsertTime = DateTime.Now;
            FrameQueueJob.Enqueue(frame);
        }

        public void FrameAborted()
        {
            OnFrameQueueComplete(new FrameQueueEventArgs(null, false));
            OnFrameQueueAction(new FrameQueueEventArgs(null, false));
        }

        #endregion Public Methods

        //**************************************************************
        #region Public Events

        public event EventHandler<FrameQueueEventArgs> FrameQueueAction;

        protected virtual void OnFrameQueueAction(FrameQueueEventArgs e)
        {
            EventHandler<FrameQueueEventArgs> handler = FrameQueueAction;
            if (handler != null)
            {
                MvvmUtils.Invoke(() => handler(this, e));
            }
        }

        public event EventHandler<FrameQueueEventArgs> FrameQueueComplete;

        protected virtual void OnFrameQueueComplete(FrameQueueEventArgs e)
        {
            EventHandler<FrameQueueEventArgs> handler = FrameQueueComplete;
            if (handler != null)
            {
                handler(this, e);
            }
        }

        #endregion Public Events

    }
}
