﻿using System;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;
using System.Text;
using Alogix.App;
using Alogix.Jobs;
using Alogix.Mvvm;

namespace FliProCameraLib
{
    /// <summary>
    /// This is the base class for FliDevice and FliDeviceManager classes
    /// It provides access to the device independent features of libflipro.
    /// </summary>
    public class FliProBase : FliPro, IDisposable
    {
        //**************************************************************
        #region Constants and Enums
        #endregion Constants and Enums

        //**************************************************************
        #region Constructor/Destructor

        //-------------------------------------------
        /// <summary>
        /// Default Constructor
        /// </summary>
        protected FliProBase()
        {
            Initialize();
        }

        #endregion Constructor/Destructor

        //**************************************************************
        #region Protected Properties
        
        #endregion

        //**************************************************************
        #region Public Properties

        //-------------------------------------------
        /// <summary>
        /// Get the current libfliPro version as a string
        /// (FliGetLibVersion)
        /// </summary>
        public string LibVersion
        {
            get
            {
                var ver = new StringBuilder(FliPro.MAX_STRING_LEN);
                ver.Append("SN_TBD");   //-=-= DEBUG
                ThrowOnError(NativeMethods.FPRO_GetLibVersion(ver, (uint)ver.MaxCapacity));
                return ver.ToString();
            }
        }

        //-------------------------------------------
        /// <summary>
        /// Get/Set the libfliPro debug log file path
        /// (FliDebugLevel)
        /// </summary>
        public string DebugFile
        {
            get { return _DebugFile; }
            set
            {
                //-=-=ThrowOnError(NativeMethods.FLISetDebugLevel(value, (int)DebugLevel));
                _DebugFile = value;
            }
        }
        private string _DebugFile;

        //-------------------------------------------
        /// <summary>
        /// Get/Set the libfli debug log level
        /// (FliDebugLevel)
        /// </summary>
        public LogLevel DebugLevel
        {
            get { return _DebugLevel; }
            set
            {
                //-=-=ThrowOnError(NativeMethods.FLISetDebugLevel(DebugFile, (int)value));
                _DebugLevel = value;
            }
        }
        private LogLevel _DebugLevel;
        #endregion

        //**************************************************************
        #region Observable Properties
        #endregion Observable Properties

        //**************************************************************
        #region Public Methods


        public void DebugLog(LogLevel level, string message)
        {
            try
            {
                //-=-=NativeMethods.FLIDebug((int)level, message);
            }
            catch (Exception)
            {
                // ignore all exceptions
                //Console.WriteLine(Ex);
            }
        }
        #endregion Public Methods

        //**************************************************************
        #region Protected Methods

        private void Initialize()
        {
            //-=-= TODO: Base configuration
        }

        /// <summary>
        /// Perform any needed cleanup prior
        /// to disposing unmanaged objects
        /// </summary>
        protected virtual void OnDispose()
        {

        }

        /// <summary>
        /// Throw exception on native function error
        /// </summary>
        /// <param name="status"> return status from Native function (negative win32 error)</param>
        /// <param name="callerName"> name of calling method or property</param>
        protected void ThrowOnError(int status, [CallerMemberName]string callerName = "")
        {
            if (status != 0)
            {
                throw new FliException(-status, callerName);
            }
        }

        #endregion Protected Methods

        //**************************************************************
        #region IDisposable Support
        private bool disposedValue = false; // To detect redundant calls

        protected virtual void Dispose(bool disposing)
        {
            if (!disposedValue)
            {
                if (disposing)
                {
                    // Dispose managed state (managed objects).
                    OnDispose();
                }

                // TODO: free unmanaged resources (unmanaged objects) and override a finalizer below.
                // TODO: set large fields to null.

                disposedValue = true;
            }
        }

        // TODO: override a finalizer only if Dispose(bool disposing) above has code to free unmanaged resources.
         ~FliProBase() {
           // Do not change this code. Put cleanup code in Dispose(bool disposing) above.
           Dispose(false);
         }

        // This code added to correctly implement the disposable pattern.
        public void Dispose()
        {
            // Do not change this code. Put cleanup code in Dispose(bool disposing) above.
            Dispose(true);
            // TODO: uncomment the following line if the finalizer is overridden above.
            // GC.SuppressFinalize(this);
        }
        #endregion

    }

}
