﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using FliUtils.Jobs;
using System.Windows.Media;

namespace FliProCameraLib
{
    /// <summary>
    /// This file contains Frame related properties and methods
    /// for the FliProCamera class
    /// 2016/09/17
    /// </summary>
    public partial class FliProCamera : FliPro
    {
        //**************************************************************
        #region Constants and Enums
        #endregion Constants and Enums

        //**************************************************************
        #region Protected Properties

        private FliProCameraManager fliProCameraManager;
        private bool ChangingExposureVisibility = false;

        #endregion Protected Properties

        //**************************************************************
        #region Public Properties
        #endregion Public Properties

        //**************************************************************
        #region Observable Properties

        //-=-= TODO: ADD The following:
        //-=-= IlluminationOnDelay
        //-=-= IlluminationOffDelay
        //-=-= IlluminationOn
        //-=-= LEDOn
        //-=-= ReadDuringExposureEnabled
        //-=-= ShutterOpenDelay
        //-=-= ShutterCloseDelay
        //-=-= ShutterOpen  ??
        //-=-= TemperatureControlEnabled
        //-=-= ThumbnailMode ??

        public string ExposureClock
        {
            get { return _ExposureClock; }
            set { SetValue<string>(ref _ExposureClock, value); }
        }
        protected string _ExposureClock = "00:00:00.010";

        //-------------------------------------------
        /// <summary>
        /// Exposure delay time in uSec
        /// </summary>
        public int ExposureDelayTime
        {
            get { return _ExposureDelayTime; }
            set
            {
                if (SetValue<int>(ref _ExposureDelayTime, value))
                {
                    if (!ChangingExposureVisibility)
                    {
                        long actualExposureTime = GetAbsoluteExposureTime(ExposureTime, ExposureUnits);
                        long actualDelayTime = GetAbsoluteExposureTime(value, ExposureDelayUnits);

                        ChangingExposureVisibility = true;

                        ExposureFrameIntervalTime = ConvertAbsoluteTimeToUnits(actualDelayTime + actualExposureTime,
                            (ExposureUnits > ExposureDelayUnits ? ExposureDelayUnits : ExposureUnits));
                        ExposureFrameIntervalUnits = (ExposureUnits > ExposureDelayUnits ? ExposureDelayUnits : ExposureUnits);

                        ChangingExposureVisibility = false;

                        SetExposure(ExposureTime, value, ExposureUnits, ExposureDelayUnits, Immediate);
                    }
                }
            }
        }
        protected int _ExposureDelayTime = 0;

        public ExposureUnits_Enum ExposureDelayUnits
        {
            get { return _ExposureDelayUnits; }
            set
            {
                if (SetValue<ExposureUnits_Enum>(ref _ExposureDelayUnits, value))
                {
                    if (!ChangingExposureVisibility)
                    {
                        // Only modify the ExposureDelayTime if it is visible. Otherwise this call came from switching visibility
                        // so all the sets are done in that function.
                        long actualExposureTime = GetAbsoluteExposureTime(ExposureTime, ExposureUnits);
                        long actualDelayTime = GetAbsoluteExposureTime(ExposureDelayTime, ExposureDelayUnits);
                        long actualFrameInterval = GetAbsoluteExposureTime(ExposureFrameIntervalTime, ExposureFrameIntervalUnits);

                        ChangingExposureVisibility = true;

                        if (actualFrameInterval != actualDelayTime + actualExposureTime)
                        {
                            // Always display the unselected time in the smallest units
                            ExposureFrameIntervalTime = ConvertAbsoluteTimeToUnits(actualExposureTime + actualDelayTime,
                                (ExposureUnits > ExposureDelayUnits ? ExposureDelayUnits : ExposureUnits));
                            ExposureFrameIntervalUnits = (ExposureUnits > ExposureDelayUnits ? ExposureDelayUnits : ExposureUnits);
                        }

                        ChangingExposureVisibility = false;

                        SetExposure(ExposureTime, ExposureDelayTime, ExposureUnits, value, Immediate);
                    }
                }
            }
        }
        protected ExposureUnits_Enum _ExposureDelayUnits = ExposureUnits_Enum.units_mSecs;

        //-------------------------------------------
        /// <summary>
        /// Exposure frame interval time in uSec
        /// </summary>
        public int ExposureFrameIntervalTime
        {
            get { return _ExposureFrameIntervalTime; }
            set
            {
                if (SetValue<int>(ref _ExposureFrameIntervalTime, value))
                {
                    if (!ChangingExposureVisibility)
                    {
                        long actualExposureTime = GetAbsoluteExposureTime(ExposureTime, ExposureUnits);
                        long actualFrameIntervalTime = GetAbsoluteExposureTime(value, ExposureFrameIntervalUnits);

                        ChangingExposureVisibility = true;
                        if (actualFrameIntervalTime < actualExposureTime)
                        {
                            actualExposureTime = actualFrameIntervalTime;
                            ExposureTime = ConvertAbsoluteTimeToUnits(actualFrameIntervalTime, ExposureFrameIntervalUnits);
                            ExposureUnits = ExposureFrameIntervalUnits;
                        }
                        ChangingExposureVisibility = false;

                        ExposureDelayTime = ConvertAbsoluteTimeToUnits(actualFrameIntervalTime - actualExposureTime, ExposureDelayUnits);
                    }
                }
            }
        }
        protected int _ExposureFrameIntervalTime = 50;

        public ExposureUnits_Enum ExposureFrameIntervalUnits
        {
            get { return _ExposureFrameIntervalUnits; }
            set
            {
                if (SetValue<ExposureUnits_Enum>(ref _ExposureFrameIntervalUnits, value))
                {
                    if (!ChangingExposureVisibility)
                    {
                        // Only modify the ExposureFrameIntervalTime if it is visible. Otherwise this call came from switching visibility
                        // so all the sets are done in that function.
                        long actualExposureTime = GetAbsoluteExposureTime(ExposureTime, ExposureUnits);
                        long actualDelayTime = GetAbsoluteExposureTime(ExposureDelayTime, ExposureDelayUnits);
                        long actualFrameIntervalTime = GetAbsoluteExposureTime(ExposureFrameIntervalTime, ExposureFrameIntervalUnits);

                        ChangingExposureVisibility = true;

                        // If the Frame Interval is less than the Exposure we need to move the Exposure down
                        if (actualFrameIntervalTime < actualExposureTime)
                        {
                            actualExposureTime = actualFrameIntervalTime;
                            ExposureTime = ConvertAbsoluteTimeToUnits(actualFrameIntervalTime, ExposureFrameIntervalUnits);
                            ExposureUnits = ExposureFrameIntervalUnits;
                        }

                        if (actualDelayTime != actualFrameIntervalTime - actualExposureTime)
                        {
                            // Always display the unselected time in the smallest units
                            ExposureDelayTime = ConvertAbsoluteTimeToUnits(actualFrameIntervalTime - actualExposureTime,
                                (ExposureUnits > ExposureFrameIntervalUnits ? ExposureFrameIntervalUnits : ExposureUnits));
                            ExposureDelayUnits = (ExposureUnits > ExposureFrameIntervalUnits ? ExposureFrameIntervalUnits : ExposureUnits);
                        }

                        ChangingExposureVisibility = false;

                        SetExposure(ExposureTime, ExposureDelayTime, ExposureUnits, ExposureDelayUnits, Immediate);
                    }
                }
            }
        }
        protected ExposureUnits_Enum _ExposureFrameIntervalUnits = ExposureUnits_Enum.units_mSecs;

        //-------------------------------------------
        /// <summary>
        /// Exposure time in uSec
        /// </summary>
        public int ExposureTime
        {
            get { return _ExposureTime; }
            set
            {
                if (SetValue<int>(ref _ExposureTime, value))
                {
                    if (!ChangingExposureVisibility)
                    {
                        long actualExposureTime = GetAbsoluteExposureTime(ExposureTime, ExposureUnits);
                        long actualDelayTime = GetAbsoluteExposureTime(ExposureDelayTime, ExposureDelayUnits);
                        long actualFrameIntervalTime = GetAbsoluteExposureTime(ExposureFrameIntervalTime, ExposureFrameIntervalUnits);

                        actualFrameIntervalTime = Math.Max(actualExposureTime, actualFrameIntervalTime);

                        ChangingExposureVisibility = true;

                        if (UseExposureDelay)
                        {
                            ExposureFrameIntervalTime = ConvertAbsoluteTimeToUnits(actualExposureTime + actualDelayTime,
                                (ExposureUnits > ExposureDelayUnits ? ExposureDelayUnits : ExposureUnits));
                            ExposureFrameIntervalUnits = (ExposureUnits > ExposureDelayUnits ? ExposureDelayUnits : ExposureUnits);
                        }
                        else
                        {
                            ExposureDelayTime = ConvertAbsoluteTimeToUnits(actualFrameIntervalTime - actualExposureTime,
                                (ExposureUnits > ExposureFrameIntervalUnits ? ExposureFrameIntervalUnits : ExposureUnits));
                            ExposureDelayUnits = (ExposureUnits > ExposureFrameIntervalUnits ? ExposureFrameIntervalUnits : ExposureUnits);
                            actualDelayTime = GetAbsoluteExposureTime(ExposureDelayTime, ExposureDelayUnits);
                            ExposureFrameIntervalTime = ConvertAbsoluteTimeToUnits(actualExposureTime + actualDelayTime,
                                (ExposureUnits > ExposureDelayUnits ? ExposureDelayUnits : ExposureUnits));
                            ExposureFrameIntervalUnits = (ExposureUnits > ExposureDelayUnits ? ExposureDelayUnits : ExposureUnits);
                        }

                        ChangingExposureVisibility = false;

                        SetExposure(value, ExposureDelayTime, ExposureUnits, ExposureDelayUnits, Immediate);
                    }
                }
            }
        }
        protected int _ExposureTime = 50;

        public ExposureUnits_Enum ExposureUnits
        {
            get { return _ExposureUnits; }
            set
            {
                if (SetValue<ExposureUnits_Enum>(ref _ExposureUnits, value))
                {
                    if (!ChangingExposureVisibility)
                    {
                        long actualExposureTime = GetAbsoluteExposureTime(ExposureTime, ExposureUnits);
                        long actualDelayTime = GetAbsoluteExposureTime(ExposureDelayTime, ExposureDelayUnits);
                        long actualFrameIntervalTime = GetAbsoluteExposureTime(ExposureFrameIntervalTime, ExposureFrameIntervalUnits);

                        ChangingExposureVisibility = true;
                        if (actualExposureTime > actualFrameIntervalTime)
                        {
                            actualFrameIntervalTime = actualExposureTime;
                            ExposureFrameIntervalTime = ConvertAbsoluteTimeToUnits(actualExposureTime, ExposureUnits);
                            ExposureFrameIntervalUnits = ExposureUnits;
                        }

                        if (UseExposureDelay)
                        {
                            ExposureFrameIntervalTime = ConvertAbsoluteTimeToUnits(actualExposureTime + actualDelayTime,
                                (ExposureUnits > ExposureDelayUnits ? ExposureDelayUnits : ExposureUnits));
                            ExposureFrameIntervalUnits = (ExposureUnits > ExposureDelayUnits ? ExposureDelayUnits : ExposureUnits);
                        }
                        else
                        {
                            ExposureDelayTime = ConvertAbsoluteTimeToUnits(actualFrameIntervalTime - actualExposureTime,
                                (ExposureUnits < ExposureFrameIntervalUnits ? ExposureUnits : ExposureFrameIntervalUnits));
                            ExposureDelayUnits = (ExposureUnits < ExposureFrameIntervalUnits ? ExposureUnits : ExposureFrameIntervalUnits);
                        }

                        ChangingExposureVisibility = false;

                        SetExposure(ExposureTime, ExposureDelayTime, value, ExposureDelayUnits, Immediate);
                    }
                }
            }
        }
        protected ExposureUnits_Enum _ExposureUnits = ExposureUnits_Enum.units_mSecs;

        public string SaveToFileActive
        {
            get { return _SaveToFileActive; }
            set { SetValue<string>(ref _SaveToFileActive, value); }
        }
        protected string _SaveToFileActive = "False";

        public Brush SaveToFileActiveTextColor
        {
            get { return _SaveToFileActiveTextColor; }
            set { SetValue(ref _SaveToFileActiveTextColor, value); }
        }
        private Brush _SaveToFileActiveTextColor = Brushes.Black;

        /// <summary>
        /// Enable/Disable Using the Exposure Delay to control frame exposure timing
        /// </summary>
        public bool UseExposureDelay
        {
            get { return _UseExposureDelay; }
            set
            {
                SetValue<bool>(ref _UseExposureDelay, value);
                if (value)
                {
                    // We need to change the delay value so that it fits in our model
                    ChangingExposureVisibility = true;

                    // First convert the displayed time into microseconds
                    long actualDelayTime = GetAbsoluteExposureTime(ExposureDelayTime, ExposureDelayUnits);

                    // Then process it to determine a good displayable time
                    ProcessExposureDelayTime(actualDelayTime);

                    long actualExposureTime = GetAbsoluteExposureTime(ExposureTime, ExposureUnits);
                    actualDelayTime = GetAbsoluteExposureTime(ExposureDelayTime, ExposureDelayUnits);

                    // Always display the unselected time in the smallest units
                    ExposureFrameIntervalTime = ConvertAbsoluteTimeToUnits(actualExposureTime + actualDelayTime,
                        (ExposureUnits > ExposureDelayUnits ? ExposureDelayUnits : ExposureUnits));
                    ExposureFrameIntervalUnits = (ExposureUnits > ExposureDelayUnits ? ExposureDelayUnits : ExposureUnits);

                    SetExposure(ExposureTime, ExposureDelayTime, ExposureUnits, ExposureDelayUnits, Immediate);

                    ChangingExposureVisibility = false;
                }
                if (UseExposureFrameInterval != !value)
                {
                    UseExposureFrameInterval = !value;
                }
            }
        }
        private bool _UseExposureDelay = true;

        /// <summary>
        /// Enable/Disable Using the Exposure Frame Interval to control frame exposure timing
        /// </summary>
        public bool UseExposureFrameInterval
        {
            get { return _UseExposureFrameInterval; }
            set
            {
                SetValue<bool>(ref _UseExposureFrameInterval, value);
                if (value)
                {
                    // We need to change the delay value so that it fits in our model
                    ChangingExposureVisibility = true;

                    // First convert the displayed time into microseconds
                    long actualFrameIntervalTime = GetTimeFromUnits(ExposureFrameIntervalTime, ExposureFrameIntervalUnits);

                    // Then process it to determine a good displayable time
                    ProcessExposureFrameIntervalTime(actualFrameIntervalTime);

                    long actualExposureTime = GetAbsoluteExposureTime(ExposureTime, ExposureUnits);
                    actualFrameIntervalTime = GetAbsoluteExposureTime(ExposureFrameIntervalTime, ExposureFrameIntervalUnits);

                    // Always display the unselected time in the smallest units
                    ExposureDelayTime = ConvertAbsoluteTimeToUnits(actualFrameIntervalTime - actualExposureTime,
                        (ExposureUnits > ExposureFrameIntervalUnits ? ExposureFrameIntervalUnits : ExposureUnits));
                    ExposureDelayUnits = (ExposureUnits > ExposureFrameIntervalUnits ? ExposureFrameIntervalUnits : ExposureUnits);

                    SetExposure(ExposureTime, ExposureDelayTime, ExposureUnits, ExposureDelayUnits, Immediate);

                    ChangingExposureVisibility = false;
                }
                if (UseExposureDelay != !value)
                {
                    UseExposureDelay = !value;
                }
            }
        }
        private bool _UseExposureFrameInterval = false;

        #endregion Observable Properties

        //**************************************************************
        #region Public Methods

        /// <summary>
        /// Converts from Absolute time back to the display time in the given units
        /// </summary>
        /// <param name="value"></param>
        /// <param name="unit"></param>
        /// <returns></returns>
        public virtual int ConvertAbsoluteTimeToUnits(long value, ExposureUnits_Enum unit)
        {
            long factor = 1;

            switch (unit)
            {
                case ExposureUnits_Enum.units_uSecs:
                    factor = 1000;
                    break;
                case ExposureUnits_Enum.units_mSecs:
                    factor = 1000000;
                    break;
                case ExposureUnits_Enum.units_secs:
                    factor = 1000000000;
                    break;
                case ExposureUnits_Enum.units_mins:
                    factor = (long)1000000000 * 60;
                    break;
                case ExposureUnits_Enum.units_hours:
                    factor = (long)1000000000 * 3600;
                    break;
            }
            value /= factor;

            return (int)value;
        }

        public virtual long GetAbsoluteExposureTime(long value, ExposureUnits_Enum unit)
        {
            long factor = 1;

            switch (unit)
            {
                case ExposureUnits_Enum.units_uSecs:
                    if ((value < 20) && (value > 0))
                    {
                        value = 20;
                    }
                    factor = 1000;
                    break;
                case ExposureUnits_Enum.units_mSecs:
                    factor = 1000000;
                    break;
                case ExposureUnits_Enum.units_secs:
                    factor = 1000000000;
                    break;
                case ExposureUnits_Enum.units_mins:
                    factor = (long)1000000000 * 60;
                    break;
                case ExposureUnits_Enum.units_hours:
                    factor = (long)1000000000 * 3600;
                    break;
            }
            value *= factor;

            return value;
        }

        public void ProcessExposureTime(long exposureTimeInMicros)
        {
            // Round the current exposure time to the nearest 100 microsecond 
            // because that is the smallest time unit the camera recognizes.
            long roundedTime = (long)Math.Round((double)exposureTimeInMicros / 100000);
            int displayTime = 0;

            // We need to limit each level to 100,000 because that is the highest number on our slider.
            // If (roundedTime % x > 0) is satisfied but (roundedTime / y > 100000) then somehow the value
            // on the board went goofy bump the units to the next level and display that value.
            // We will only return when we find the right units to display
            if (roundedTime > 0)
            {
                if (roundedTime % 10 > 0)
                {
                    displayTime = (int)roundedTime * 100;
                    if (displayTime <= 100000)
                    {
                        // Display the time in Microseconds
                        ExposureTime = displayTime;
                        ExposureUnits = ExposureUnits_Enum.units_uSecs;
                        return;
                    }
                }

                if (roundedTime % 10000 > 0)
                {
                    displayTime = (int)roundedTime / 10;
                    if (displayTime <= 100000)
                    {
                        // Display the time in Milliseconds
                        ExposureTime = displayTime;
                        ExposureUnits = ExposureUnits_Enum.units_mSecs;
                        return;
                    }
                }

                if (roundedTime % 600000 > 0)
                {
                    displayTime = (int)roundedTime / 10000;
                    if (displayTime <= 100000)
                    {
                        // Display the time in Seconds
                        ExposureTime = displayTime;
                        ExposureUnits = ExposureUnits_Enum.units_secs;
                        return;
                    }
                }

                if (roundedTime % 36000000 > 0)
                {
                    displayTime = (int)roundedTime / 600000;
                    if (displayTime <= 100000)
                    {
                        // Display the time in Minutes
                        ExposureTime = displayTime;
                        ExposureUnits = ExposureUnits_Enum.units_mins;
                        return;
                    }
                }

                if (roundedTime / 36000000 <= 100000)
                {
                    displayTime = (int)roundedTime / 36000000;
                    if (displayTime <= 100000)
                    {
                        // Display the time in Hours
                        ExposureTime = displayTime;
                        ExposureUnits = ExposureUnits_Enum.units_hours;
                        return;
                    }
                }
            }

            // else bad data coming in so set the value to the default
            ExposureTime = 0;
            ExposureUnits = ExposureUnits_Enum.units_mSecs;
        }

        public void ProcessExposureDelayTime(long exposureDelayTimeInMicros)
        {
            // Round the current exposure time to the nearest 100 microsecond 
            // because that is the smallest time unit the camera recognizes.
            long roundedTime = (long)Math.Round((double)exposureDelayTimeInMicros / 100000);
            int displayTime = 0;

            // We need to limit each level to 100,000 because that is the highest number on our slider.
            // If (roundedTime % x > 0) is satisfied but (roundedTime / y > 100000) then somehow the value
            // on the board went goofy bump the units to the next level and display that value.
            // We will only return when we find the right units to display
            if (roundedTime > 0)
            {
                if (roundedTime % 10 > 0)
                {
                    displayTime = (int)roundedTime * 100;
                    if (displayTime <= 100000)
                    {
                        // Display the time in Microseconds
                        ExposureDelayTime = displayTime;
                        ExposureDelayUnits = ExposureUnits_Enum.units_uSecs;
                        return;
                    }
                }

                if (roundedTime % 10000 > 0)
                {
                    displayTime = (int)roundedTime / 10;
                    if (displayTime <= 100000)
                    {
                        // Display the time in Milliseconds
                        ExposureDelayTime = displayTime;
                        ExposureDelayUnits = ExposureUnits_Enum.units_mSecs;
                        return;
                    }
                }

                if (roundedTime % 600000 > 0)
                {
                    displayTime = (int)roundedTime / 10000;
                    if ((displayTime <= 100000) && (displayTime % 60 > 0))
                    {
                        // Display the time in Seconds
                        ExposureDelayTime = displayTime;
                        ExposureDelayUnits = ExposureUnits_Enum.units_secs;
                        return;
                    }
                }

                if (roundedTime % 36000000 > 0)
                {
                    displayTime = (int)roundedTime / 600000;
                    if ((displayTime <= 100000) && (displayTime % 60 > 0))
                    {
                        // Display the time in Minutes
                        ExposureDelayTime = displayTime;
                        ExposureDelayUnits = ExposureUnits_Enum.units_mins;
                        return;
                    }
                }

                if (roundedTime / 36000000 <= 100000)
                {
                    displayTime = (int)roundedTime / 36000000;
                    if ((displayTime <= 100000) && (displayTime % 100 > 0))
                    {
                        // Display the time in Hours
                        ExposureDelayTime = (int)roundedTime / 36000000;
                        ExposureDelayUnits = ExposureUnits_Enum.units_hours;
                        return;
                    }
                }
            }

            // else bad data coming in so set the value to the default
            //ExposureDelayTime = 0;
            //ExposureDelayUnits = ExposureUnits_Enum.units_mSecs;
        }

        public void ProcessExposureFrameIntervalTime(long exposureFrameIntervalTimeInMicros)
        {
            // Round the current exposure time to the nearest 100 microsecond 
            // because that is the smallest time unit the camera recognizes.
            long roundedTime = (long)Math.Round((double)exposureFrameIntervalTimeInMicros / 100);
            int displayTime = 0;

            // We need to limit each level to 100,000 because that is the highest number on our slider.
            // If (roundedTime % x > 0) is satisfied but (roundedTime / y > 100000) then somehow the value
            // on the board went goofy bump the units to the next level and display that value.
            // We will only return when we find the right units to display
            if (roundedTime > 0)
            {
                if (roundedTime % 10 > 0)
                {
                    displayTime = (int)roundedTime * 100;
                    if (displayTime <= 100000)
                    {
                        // Display the time in Microseconds
                        ExposureFrameIntervalTime = displayTime;
                        ExposureFrameIntervalUnits = ExposureUnits_Enum.units_uSecs;
                        return;
                    }
                }

                if (roundedTime % 10000 > 0)
                {
                    displayTime = (int)roundedTime / 10;
                    if (displayTime <= 100000)
                    {
                        // Display the time in Milliseconds
                        ExposureFrameIntervalTime = displayTime;
                        ExposureFrameIntervalUnits = ExposureUnits_Enum.units_mSecs;
                        return;
                    }
                }

                if (roundedTime % 600000 > 0)
                {
                    displayTime = (int)roundedTime / 10000;
                    if ((displayTime <= 100000) && (displayTime % 60 > 0))
                    {
                        // Display the time in Seconds
                        ExposureFrameIntervalTime = displayTime;
                        ExposureFrameIntervalUnits = ExposureUnits_Enum.units_secs;
                        return;
                    }
                }

                if (roundedTime % 36000000 > 0)
                {
                    displayTime = (int)roundedTime / 600000;
                    if ((displayTime <= 100000) && (displayTime % 60 > 0))
                    {
                        // Display the time in Minutes
                        ExposureFrameIntervalTime = displayTime;
                        ExposureFrameIntervalUnits = ExposureUnits_Enum.units_mins;
                        return;
                    }
                }

                if (roundedTime / 36000000 <= 100000)
                {
                    displayTime = (int)roundedTime / 36000000;
                    if ((displayTime <= 100000) && (displayTime % 100 > 0))
                    {
                        // Display the time in Hours
                        ExposureFrameIntervalTime = displayTime;
                        ExposureFrameIntervalUnits = ExposureUnits_Enum.units_hours;
                        return;
                    }
                }
            }

            // else bad data coming in so set the value to the max time
            ExposureFrameIntervalTime = 11;
            ExposureFrameIntervalUnits = ExposureUnits_Enum.units_hours;
        }

        public void SetSaveToFileActive()
        {
            if (fliProCameraManager == null)
            {
                fliProCameraManager = FliProCameraManager.Instance;
            }
            if (!String.IsNullOrEmpty(fliProCameraManager.RawFilePath))
            {
                SaveToFileActiveTextColor = Brushes.Red;
                SaveToFileActive = "True";
            }
            else
            {
                SaveToFileActiveTextColor = Brushes.Black;
                SaveToFileActive = "False";
            }
        }

        #endregion Public Methods

        //**************************************************************
        #region Protected Methods

        protected virtual void SetExposure(long value, long delay, ExposureUnits_Enum unit, ExposureUnits_Enum unitDelay, bool Immediate) { }

        #endregion Protected Methods

    }
}
